# Export the data from a timeseries netcdf file 

# name of the file
filename <- "YOURFILENAME.nc"
# Name of the variable in the netcdf-file
varname <- "Data1"

# name of the outputfile
outfilename <- "outfile.csv"

# For Cloud Type (CTY) the following varnames are allowed:
# Data1 = Low Clouds, Data2 = Middle level clouds, Data3 = High opaque clouds
# Data4 = High Semitransparent clouds, Data5 = Fractional Clouds

# For HLW and HSH select the variable and the vertical level
# HLW: level: 1-5, varname: LPWm, RHWm, Tm
# HSH: level: 1-6, varname: Qm, Tm
level <- 5

# The following information is required to generate the name of the directory
# Select the name of the Products/DataSets
product <- "SIS"
# type = Products or DataSets
type <- "Products"

# This directory needs to be adjusted only once
dir <- "C:/Dokumente und Einstellungen/uwpfeifr/Desktop/CMSAF_neu/Data"
#dir <- "C:/Dokumente und Einstellungen/jtrentma/Desktop/CMSAF_V2/Data"

#----------------------------------------------------------------
# NO CHANGES REQUIRED BEYOND THIS POINT
#----------------------------------------------------------------

datadir <- paste(dir,type,product,"",sep="/")

file <- paste(datadir,filename,sep="")
outfile <- paste(datadir,outfilename,sep="")

# make sure you have the ncdf- and RNetCDF-packages installed on your R-installation!!
library(ncdf)
library(RNetCDF)
library(Hmisc)

# Open the netcdf-file
nc <- open.ncdf(file)

# Read the available variable in the file
nvars <- nc$nvars
varnames <- vector(mode="character",length=nvars)
for (n in 1:nvars) {
	varnames[n] <- nc$var[[n]]$name
}
if (varname %nin% varnames) {
print("Error, variable not in file!")
print("The following variables are available for selection:")
print(varnames)
stop("Please select one of the above variable! (varname)")
}

# In HLW and HSH multiple variables are stored in each file
if (product == "HLW" || product =="HSH") {
	varname <- paste(product,"_",variable,sep="")
	}
	
# Read in the data
field <- get.var.ncdf(nc, varname)
if (product == "HLW" || product =="HSH") {
	field <- field[level,]
	}	
	
unit <- att.get.ncdf(nc, varname,"units")$value

missval <- att.get.ncdf(nc,varname,"_FillValue")$value
# The offset and the scalefactor is required because
# the Fill_Value attribue is not applied by the ncdf-package
# The offset and scalefactor is automatically applied to all data
offset.value <- 0.
scale.factor <- 1.
if (att.get.ncdf(nc,varname,"add_offset")$hasatt==TRUE) {
	offset.value <- att.get.ncdf(nc,varname,"add_offset")$value }
if (att.get.ncdf(nc,varname,"scale_factor")$hasatt==TRUE) {
	scale.factor <- att.get.ncdf(nc,varname,"scale_factor")$value }

# Close the file
close.ncdf(nc)

# Set the missing data to NA
# Since the data has been corrected for the scalefactor and the offset, 
# this has also to be considered for the FillValue
na.ind <- which(field== (missval*scale.factor + offset.value))
field[na.ind] <- NA

#--------------------------------------------------#

# determine the location 
londim <- nc$dim[["lon"]]
lon <- londim$vals
latdim <- nc$dim[["lat"]]
lat <- latdim$vals

#--------------------------------------------------#

# retrieve the time variable
timedim <- nc$dim[["time"]]
nt <- timedim$len
time.unit <- timedim$units
time <- timedim$vals

# Create a R-date-object 
date.time <- as.Date(utcal.nc(time.unit,time,type="s"))

#--------------------------------------------------#

out <- data.frame(date.time,field)

write.csv2(out, file = outfile)