# This script displays a map of the selected product 
# You can either specify a certain year / month from a data file with several time steps
# or plot one 2D field 

# name of the file (NorthPole projection)
filename <- "YOURFILENAME.nc"
# Name of the variable in the netcdf-file
varname <- "sal"

# For Cloud Type (CTY) the following varnames are allowed:
# Data1 = Low Clouds, Data2 = Middle level clouds, Data3 = High opaque clouds
# Data4 = High Semitransparent clouds, Data5 = Fractional Clouds

# For HLW and HSH select the variable and the vertical level
# HLW: level: 1-5, varname: LPWm, RHWm, Tm
# HSH: level: 1-6, varname: Qm, Tm
level <- 5

# The following information is required to generate the name of the directory
# Select the name of the Products/DataSets
product <- "SAL"
# type = Products or DataSets
type <- "DataSets"

# If you want to create a png-file, you need to specify the filename
picout <- FALSE
picfile <- "YOURPICFILE.png"

# In case you want to add something to the title of the plot
add2title <- "CM SAF, "

# for files with multiple times: select the time
# Make sure that the selected time is available in the data file
year <- 2012
month <- 4

# here you may define your OWN DATE in the image-title (eg in case of seasonal data)
own_date <- FALSE
owndate <- "DJF 2007/2008/2009"

# if you want to choose the TIMESTEP instead of year and month,
# then set select_timestep="TRUE" (uncomment the following line and choose the timestep)
select_timestep <- FALSE
tstep <- 1

# Set the size of the output window (in # of pixels)
height <- 600
width <- 600

# Select to plot an anomaly (use of appropriate color scale)
plot.ano <- FALSE

# Set the breaks of the color bar manually
# Suggested for plotting of anomalies
set.col.breaks <- FALSE
#brk.set <- c(-50,-20,-10,-5,-2.5,2.5,5,10,20,50)
#brk.set <- c(0,50,55,60,65,70,75,80,85,90)
brk.set <- seq(0,90,5)

# select the range of the color bar
# select -99 for flexible range
colmin0 <- -99
colmax0 <- -99
# Set the number of colors
ncol <- 14

# Set to true if higher resolution additional data is required
plotHighRes <- FALSE
plotVeryHighRes <- FALSE

# Switches to determine additional plot layers
plotCoastline <- TRUE
plotCountries <- TRUE
plotRivers <- FALSE
plotLakes <- FALSE
# Thickness of the additional lines
contour.thick <- 2.

# Switches to control the display of cities
plotCities <- FALSE
# Set to true if more cities should be displayed
plotCitiesHighRes <- FALSE
plotCitiesVeryHighRes <- FALSE
# symbol
pch.cities <- 2
# size of symbol
cex.cities <- 1.
# name next to the sysmbol?
label.cities <- TRUE
# size of the label
cex.label.cities <- 0.5
# lat offset of label
dlat <- 0.25

# Set the main data directoy 
dir <- "YOURDATADIRECTORY"
#dir <- "C:/Dokumente und Einstellungen/uwpfeifr/Desktop/CMSAF_neu/Data/"

#----------------------------------------------------------------
# NO CHANGES REQUIRED BEYOND THIS POINT
#----------------------------------------------------------------

datadir <- paste(dir,type,product,"",sep="/")
picdir <- paste(datadir,"pics/",sep="/")
file <- paste(datadir,filename,sep="")

# make sure that you have the ncdf-, the fields- and the maps-package installed! 
library(ncdf)
library(RNetCDF)
library(fields)
library(maptools)
library(RColorBrewer)
library(colorRamps)
library(sp)
library(raster)
library(rasterVis)
library(rgdal)
library(Hmisc)

CRS.arctic <- CRS("+proj=laea +a=6370997.0 +lon_0=0. +lat_0=90.")

if (plotCoastline) {
aux.file <- "110m_coastline.shp"
if (plotHighRes) aux.file <- "50m_coastline.shp"
if (plotVeryHighRes) aux.file <- "10m_coastline.shp"
coastline.file <- paste(dir,"AuxiliaryData/coastline/",aux.file,sep="")
coastline <- readShapeSpatial(coastline.file)
coastline <- as(coastline,"SpatialLines")
proj4string(coastline) <- CRS("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")
coastline.arctic <- spTransform(coastline,CRS.arctic)
}

if (plotCountries) {
aux.file <- "ne_110m_admin_0_countries.shp"
if (plotHighRes) aux.file <- "ne_50m_admin_0_countries.shp"
if (plotVeryHighRes) aux.file <- "ne_10m_admin_0_countries.shp"
countries.file <- paste(dir,"AuxiliaryData/countries/",aux.file,sep="")
countries <- readShapeSpatial(countries.file)
countries <- as(countries,"SpatialLines")
proj4string(countries) <- CRS("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")
countries.arctic <- spTransform(countries,CRS.arctic)
}

if (plotRivers) {
aux.file <- "110m-rivers-lake-centerlines.shp"
if (plotHighRes) aux.file <- "50m-rivers-lake-centerlines.shp"
if (plotVeryHighRes) aux.file <- "ne_10m_rivers_lake_centerlines.shp"
rivers.file <- paste(dir,"AuxiliaryData/rivers/",aux.file,sep="")
rivers <- readShapeSpatial(rivers.file)
rivers <- as(rivers,"SpatialLines")
proj4string(rivers) <- CRS("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")
rivers.arctic <- spTransform(rivers,CRS.arctic)
}

if (plotLakes) {
aux.file <- "110m_lakes.shp"
if (plotHighRes) aux.file <- "50m-lakes.shp"
if (plotVeryHighRes) aux.file <- "ne_10m_lakes.shp"
lakes.file <- paste(dir,"AuxiliaryData/lakes/",aux.file,sep="")
lakes <- readShapeSpatial(lakes.file)
lakes <- as(lakes,"SpatialLines")
proj4string(lakes) <- CRS("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs")
lakes.arctic <- spTransform(lakes,CRS.arctic)
}

if (plotCities) {
aux.file <- "ne_110m_populated_places.shp"
if (plotCitiesHighRes) aux.file <- "ne_50m_populated_places.shp"
if (plotCitiesVeryHighRes) aux.file <- "ne_10m_populated_places.shp"
cities.file <- paste(dir,"AuxiliaryData/populated-places/",aux.file,sep="")
cities <- readShapeSpatial(cities.file)
lon.cities <- cities@data$LONGITUDE
lat.cities <- cities@data$LATITUDE
names.cities <- cities@data$NAME

}

# Open the netcdf-file
nc <- open.ncdf(file)

# Read the available variable in the file
nvars <- nc$nvars
varnames <- vector(mode="character",length=nvars)
for (n in 1:nvars) {
	varnames[n] <- nc$var[[n]]$name
}
if (varname %nin% varnames) {
print("Error, variable not in file!")
print("The following variables are available for selection:")
print(varnames)
stop("Please select one of the above variable! (varname)")
}

# Retrieve the unit, the missing_data-value, and the title of the data
unit <- att.get.ncdf(nc, varname,"units")$value
missval <- att.get.ncdf(nc,varname,"_FillValue")$value

if (att.get.ncdf(nc,varname,"title")$hasatt==TRUE) {
	name <- att.get.ncdf(nc,varname,"title")$value 
	} else {
	 name <- varname
	}
	
# The offset and the scalefactor is required because
# the Fill_Value attribute is not applied by the ncdf-package
# The offset and scalefactor is automatically applied to all data
offset.value <- 0.
scale.factor <- 1.
if (att.get.ncdf(nc,varname,"add_offset")$hasatt==TRUE) {
	offset.value <- att.get.ncdf(nc,varname,"add_offset")$value }
if (att.get.ncdf(nc,varname,"scale_factor")$hasatt==TRUE) {
	scale.factor <- att.get.ncdf(nc,varname,"scale_factor")$value }

#--------------------------------------------------#

# determine the grid
xdim <- nc$dim[["x"]]
#print(c(xdim,ydim))
x <- xdim$vals
nx <- xdim$len
ydim <- nc$dim[["y"]]
y <- ydim$vals
ny <- ydim$len

#--------------------------------------------------#

# retrieve the time variable
timedim <- nc$dim[["time"]]
nt <- timedim$len
time.unit <- timedim$units
time <- timedim$vals

# Create a R-date-object 
date.time <- as.Date(utcal.nc(time.unit,time,type="s"))

itime <- 1
if (nt > 1) {
# determine the index of the selected time step
  timeout <- as.Date(paste(year,"-",month,"-1",sep=""))
  itime <- which(date.time==timeout)

# check if the time is included in the file
  if (length(itime) == 0) stop("Selected time not included in the file!")
  }
  
# if timestep is used instead of "year" and "month" ...
if (select_timestep== TRUE) {itime<- tstep}   
  
# Read in the data 
field <- get.var.ncdf(nc, varname,start=c(1,1,itime),count=c(nx,ny,1))	
# Set the missing data to NA
# Since the data has been corrected for the scalefactor and the offset, 
# this has also to be considered for the FillValue
  na.ind <- which(field==(missval*scale.factor + offset.value))
  field[na.ind] <- NA
		
# Close the file
  close.ncdf(nc)

# format the time step for plotting
if (own_date != TRUE)
{
if (nt > 1) {
  zdate <- paste(format(date.time[itime],"%Y%m"),sep=" ")
  if (nt == 12) zdate <- paste(format(date.time[itime],"%m"),sep=" ")
	} else zdate <- ""
}
if (own_date == TRUE) {zdate <- owndate}

# Set the title of the plot
  title <- paste(name," (",unit,"), ",sep="")
  if (product == "HLW" || product =="HSH") {
	title <- paste(varname," (",unit,"), level: ",level.out,", ",sep="")
	}
	
colmin <- colmin0
colmax <- colmax0

if ((as.integer(colmin) == -99) && (as.integer(colmax) == -99)) {
  colmin <- min(field,na.rm=TRUE)
  colmax <- max(field,na.rm=TRUE)
}

if (set.col.breaks) {
  brk <- brk.set
} else {
  brk <- seq(colmin,colmax,length.out=ncol+1)
} 

# Set the colors and the color bar for the Difference plots
col.breaks <- brk
ncolor <- length(col.breaks)
at.ticks <- seq(1,ncolor)
names.ticks <- col.breaks[at.ticks]
zlim <- c(1,ncolor)

colors <- matlab.like(ncolor-1)
if (plot.ano) colors[as.integer(ncolor/2)] <- rgb(1,1,1)

# Generate the raster object
if (type == "Products") {
	x1 <- -2497500.0 } else {
	x1 <- -2487500.0
}
y1 <- x1
x2 <- -x1
y2 <- x2
x <- seq(x1,x2,length=nx)
y <- seq(y1,y2,length=ny)
#print(c(x,y))
grd <- expand.grid(x=x,y=y)
coordinates(grd) <- ~x+y
gridded(grd) <- TRUE
proj4string(grd) <- CRS.arctic
grd.sp <- getGridTopology(grd)
fieldf <- as.vector(field)

# Generate the field to be plotted
field.plot <- vector(mode="numeric",length=length(fieldf))
for (l in seq(1,ncolor-1) ) {
  idx <- which(fieldf >= col.breaks[l] &
               fieldf < col.breaks[l+1])
  field.plot[idx] <- l - 0.1
}
field.sp <- SpatialGridDataFrame(grd.sp,as.data.frame(fieldf),proj4string=CRS.arctic)
field.plot.sp <- SpatialGridDataFrame(grd.sp,as.data.frame(field.plot),proj4string=CRS.arctic)

raster.data <- raster(field.sp,layer=1,values=TRUE)
raster.data.plot <- raster(field.plot.sp,layer=1,values=TRUE)
raster.data <- addLayer(raster.data,raster.data.plot)

#----------------------------------------------------------
# Open a png-file
 
if (picout == TRUE) png(file=paste(picdir,picfile,sep=""),height=height,width=width)

# set the number of rows and columns of the plot
par(mfrow = c(1,1))

myTheme <- rasterTheme(region=colors)
#myTheme <- GrTheme
myColorkey <- list(at=col.breaks,labels=list(labels=names.ticks,at=col.breaks))
#myColorkey <- list(at=at.ticks,labels=list(
#	labels=names.ticks,at=at.ticks
#	))
cmsaf.plot <- levelplot(raster.data,layer=1,margin=FALSE,par.settings=myTheme,main=paste(title,add2title,zdate,sep=""),
	at=col.breaks,colorkey=myColorkey,ylab="",scales=list(draw=FALSE)) + 
	layer({
		if (plotCoastline) sp.lines(coastline.arctic, lwd=contour.thick)
		if (plotCountries) sp.lines(countries.arctic, lwd=contour.thick)
		if (plotRivers) sp.lines(rivers.arctic, lwd=contour.thick,col="blue")
		if (plotLakes) sp.lines(lakes.arctic, lwd=contour.thick,col="blue")
		})

print(cmsaf.plot)
# close the png-file
if (picout == TRUE) dev.off()
